@@ -12,27 +12,39 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-#ifndef SPIRV_TOOLS_IR_CONTEXT_H
-#define SPIRV_TOOLS_IR_CONTEXT_H
-
-#include "assembly_grammar.h"
-#include "cfg.h"
-#include "constants.h"
-#include "decoration_manager.h"
-#include "def_use_manager.h"
-#include "dominator_analysis.h"
-#include "feature_manager.h"
-#include "loop_descriptor.h"
-#include "module.h"
-#include "type_manager.h"
+#ifndef SOURCE_OPT_IR_CONTEXT_H_
+#define SOURCE_OPT_IR_CONTEXT_H_
 
 #include <algorithm>
 #include <iostream>
 #include <limits>
+#include <map>
+#include <memory>
+#include <queue>
+#include <unordered_map>
 #include <unordered_set>
+#include <utility>
+#include <vector>
+
+#include "source/assembly_grammar.h"
+#include "source/opt/cfg.h"
+#include "source/opt/constants.h"
+#include "source/opt/decoration_manager.h"
+#include "source/opt/def_use_manager.h"
+#include "source/opt/dominator_analysis.h"
+#include "source/opt/feature_manager.h"
+#include "source/opt/fold.h"
+#include "source/opt/loop_descriptor.h"
+#include "source/opt/module.h"
+#include "source/opt/register_pressure.h"
+#include "source/opt/scalar_analysis.h"
+#include "source/opt/struct_cfg_analysis.h"
+#include "source/opt/type_manager.h"
+#include "source/opt/value_number_table.h"
+#include "source/util/make_unique.h"
 
 namespace spvtools {
-namespace ir {
+namespace opt {
 
 class IRContext {
  public:
@@ -58,16 +70,26 @@ class IRContext {
     kAnalysisDominatorAnalysis = 1 << 5,
     kAnalysisLoopAnalysis = 1 << 6,
     kAnalysisNameMap = 1 << 7,
-    kAnalysisEnd = 1 << 8
+    kAnalysisScalarEvolution = 1 << 8,
+    kAnalysisRegisterPressure = 1 << 9,
+    kAnalysisValueNumberTable = 1 << 10,
+    kAnalysisStructuredCFG = 1 << 11,
+    kAnalysisBuiltinVarId = 1 << 12,
+    kAnalysisIdToFuncMapping = 1 << 13,
+    kAnalysisConstants = 1 << 14,
+    kAnalysisTypes = 1 << 15,
+    kAnalysisEnd = 1 << 16
   };
 
+  using ProcessFunction = std::function<bool(Function*)>;
+
   friend inline Analysis operator|(Analysis lhs, Analysis rhs);
   friend inline Analysis& operator|=(Analysis& lhs, Analysis rhs);
   friend inline Analysis operator<<(Analysis a, int shift);
   friend inline Analysis& operator<<=(Analysis& a, int shift);
 
   // Creates an |IRContext| that contains an owned |Module|
-  IRContext(spv_target_env env, spvtools::MessageConsumer c)
+  IRContext(spv_target_env env, MessageConsumer c)
       : syntax_context_(spvContextCreate(env)),
         grammar_(syntax_context_),
         unique_id_(0),
@@ -77,13 +99,15 @@ class IRContext {
         valid_analyses_(kAnalysisNone),
         constant_mgr_(nullptr),
         type_mgr_(nullptr),
-        id_to_name_(nullptr) {
-    libspirv::SetContextMessageConsumer(syntax_context_, consumer_);
+        id_to_name_(nullptr),
+        max_id_bound_(kDefaultMaxIdBound),
+        preserve_bindings_(false),
+        preserve_spec_constants_(false) {
+    SetContextMessageConsumer(syntax_context_, consumer_);
     module_->SetContext(this);
   }
 
-  IRContext(spv_target_env env, std::unique_ptr<Module>&& m,
-            spvtools::MessageConsumer c)
+  IRContext(spv_target_env env, std::unique_ptr<Module>&& m, MessageConsumer c)
       : syntax_context_(spvContextCreate(env)),
         grammar_(syntax_context_),
         unique_id_(0),
@@ -92,8 +116,11 @@ class IRContext {
         def_use_mgr_(nullptr),
         valid_analyses_(kAnalysisNone),
         type_mgr_(nullptr),
-        id_to_name_(nullptr) {
-    libspirv::SetContextMessageConsumer(syntax_context_, consumer_);
+        id_to_name_(nullptr),
+        max_id_bound_(kDefaultMaxIdBound),
+        preserve_bindings_(false),
+        preserve_spec_constants_(false) {
+    SetContextMessageConsumer(syntax_context_, consumer_);
     module_->SetContext(this);
     InitializeCombinators();
   }
@@ -120,8 +147,8 @@ class IRContext {
   inline IteratorRange<Module::const_inst_iterator> capabilities() const;
 
   // Iterators for types, constants and global variables instructions.
-  inline ir::Module::inst_iterator types_values_begin();
-  inline ir::Module::inst_iterator types_values_end();
+  inline Module::inst_iterator types_values_begin();
+  inline Module::inst_iterator types_values_end();
   inline IteratorRange<Module::inst_iterator> types_values();
   inline IteratorRange<Module::const_inst_iterator> types_values() const;
 
@@ -197,16 +224,43 @@ class IRContext {
 
   // Returns a pointer to a def-use manager.  If the def-use manager is
   // invalid, it is rebuilt first.
-  opt::analysis::DefUseManager* get_def_use_mgr() {
+  analysis::DefUseManager* get_def_use_mgr() {
     if (!AreAnalysesValid(kAnalysisDefUse)) {
       BuildDefUseManager();
     }
     return def_use_mgr_.get();
   }
 
+  // Returns a pointer to a value number table.  If the liveness analysis is
+  // invalid, it is rebuilt first.
+  ValueNumberTable* GetValueNumberTable() {
+    if (!AreAnalysesValid(kAnalysisValueNumberTable)) {
+      BuildValueNumberTable();
+    }
+    return vn_table_.get();
+  }
+
+  // Returns a pointer to a StructuredCFGAnalysis.  If the analysis is invalid,
+  // it is rebuilt first.
+  StructuredCFGAnalysis* GetStructuredCFGAnalysis() {
+    if (!AreAnalysesValid(kAnalysisStructuredCFG)) {
+      BuildStructuredCFGAnalysis();
+    }
+    return struct_cfg_analysis_.get();
+  }
+
+  // Returns a pointer to a liveness analysis.  If the liveness analysis is
+  // invalid, it is rebuilt first.
+  LivenessAnalysis* GetLivenessAnalysis() {
+    if (!AreAnalysesValid(kAnalysisRegisterPressure)) {
+      BuildRegPressureAnalysis();
+    }
+    return reg_pressure_.get();
+  }
+
   // Returns the basic block for instruction |instr|. Re-builds the instruction
   // block map, if needed.
-  ir::BasicBlock* get_instr_block(ir::Instruction* instr) {
+  BasicBlock* get_instr_block(Instruction* instr) {
     if (!AreAnalysesValid(kAnalysisInstrToBlockMapping)) {
       BuildInstrToBlockMapping();
     }
@@ -218,14 +272,14 @@ class IRContext {
   // needed.
   //
   // |id| must be a registered definition.
-  ir::BasicBlock* get_instr_block(uint32_t id) {
-    ir::Instruction* def = get_def_use_mgr()->GetDef(id);
+  BasicBlock* get_instr_block(uint32_t id) {
+    Instruction* def = get_def_use_mgr()->GetDef(id);
     return get_instr_block(def);
   }
 
   // Sets the basic block for |inst|. Re-builds the mapping if it has become
   // invalid.
-  void set_instr_block(ir::Instruction* inst, ir::BasicBlock* block) {
+  void set_instr_block(Instruction* inst, BasicBlock* block) {
     if (AreAnalysesValid(kAnalysisInstrToBlockMapping)) {
       instr_to_block_[inst] = block;
     }
@@ -233,31 +287,42 @@ class IRContext {
 
   // Returns a pointer the decoration manager.  If the decoration manger is
   // invalid, it is rebuilt first.
-  opt::analysis::DecorationManager* get_decoration_mgr() {
+  analysis::DecorationManager* get_decoration_mgr() {
     if (!AreAnalysesValid(kAnalysisDecorations)) {
       BuildDecorationManager();
     }
     return decoration_mgr_.get();
-  };
+  }
 
   // Returns a pointer to the constant manager.  If no constant manager has been
   // created yet, it creates one.  NOTE: Once created, the constant manager
   // remains active and it is never re-built.
-  opt::analysis::ConstantManager* get_constant_mgr() {
-    if (!constant_mgr_)
-      constant_mgr_.reset(new opt::analysis::ConstantManager(this));
+  analysis::ConstantManager* get_constant_mgr() {
+    if (!AreAnalysesValid(kAnalysisConstants)) {
+      BuildConstantManager();
+    }
     return constant_mgr_.get();
   }
 
   // Returns a pointer to the type manager.  If no type manager has been created
   // yet, it creates one. NOTE: Once created, the type manager remains active it
   // is never re-built.
-  opt::analysis::TypeManager* get_type_mgr() {
-    if (!type_mgr_)
-      type_mgr_.reset(new opt::analysis::TypeManager(consumer(), this));
+  analysis::TypeManager* get_type_mgr() {
+    if (!AreAnalysesValid(kAnalysisTypes)) {
+      BuildTypeManager();
+    }
     return type_mgr_.get();
   }
 
+  // Returns a pointer to the scalar evolution analysis. If it is invalid it
+  // will be rebuilt first.
+  ScalarEvolutionAnalysis* GetScalarEvolutionAnalysis() {
+    if (!AreAnalysesValid(kAnalysisScalarEvolution)) {
+      BuildScalarEvolutionAnalysis();
+    }
+    return scalar_evolution_analysis_.get();
+  }
+
   // Build the map from the ids to the OpName and OpMemberName instruction
   // associated with it.
   inline void BuildIdToNameMap();
@@ -269,12 +334,10 @@ class IRContext {
 
   // Sets the message consumer to the given |consumer|. |consumer| which will be
   // invoked every time there is a message to be communicated to the outside.
-  void SetMessageConsumer(spvtools::MessageConsumer c) {
-    consumer_ = std::move(c);
-  }
+  void SetMessageConsumer(MessageConsumer c) { consumer_ = std::move(c); }
 
   // Returns the reference to the message consumer for this pass.
-  const spvtools::MessageConsumer& consumer() const { return consumer_; }
+  const MessageConsumer& consumer() const { return consumer_; }
 
   // Rebuilds the analyses in |set| that are invalid.
   void BuildInvalidAnalyses(Analysis set);
@@ -306,7 +369,7 @@ class IRContext {
   //
   // Returns a pointer to the instruction after |inst| or |nullptr| if no such
   // instruction exists.
-  Instruction* KillInst(ir::Instruction* inst);
+  Instruction* KillInst(Instruction* inst);
 
   // Returns true if all of the given analyses are valid.
   bool AreAnalysesValid(Analysis set) { return (set & valid_analyses_) == set; }
@@ -319,6 +382,15 @@ class IRContext {
   // |before| and |after| must be registered definitions in the DefUseManager.
   bool ReplaceAllUsesWith(uint32_t before, uint32_t after);
 
+  // Replace all uses of |before| id with |after| id if those uses
+  // (instruction, operand pair) return true for |predicate|. Returns true if
+  // any replacement happens. This method does not kill the definition of the
+  // |before| id. If |after| is the same as |before|, does nothing and return
+  // false.
+  bool ReplaceAllUsesWithPredicate(
+      uint32_t before, uint32_t after,
+      const std::function<bool(Instruction*, uint32_t)>& predicate);
+
   // Returns true if all of the analyses that are suppose to be valid are
   // actually valid.
   bool IsConsistent();
@@ -340,7 +412,7 @@ class IRContext {
   void KillNamesAndDecorates(uint32_t id);
 
   // Kill all name and decorate ops targeting the result id of |inst|.
-  void KillNamesAndDecorates(ir::Instruction* inst);
+  void KillNamesAndDecorates(Instruction* inst);
 
   // Returns the next unique id for use by an instruction.
   inline uint32_t TakeNextUniqueId() {
@@ -352,7 +424,7 @@ class IRContext {
 
   // Returns true if |inst| is a combinator in the current context.
   // |combinator_ops_| is built if it has not been already.
-  inline bool IsCombinatorInstruction(ir::Instruction* inst) {
+  inline bool IsCombinatorInstruction(const Instruction* inst) {
     if (!AreAnalysesValid(kAnalysisCombinators)) {
       InitializeCombinators();
     }
@@ -369,7 +441,7 @@ class IRContext {
   }
 
   // Returns a pointer to the CFG for all the functions in |module_|.
-  ir::CFG* cfg() {
+  CFG* cfg() {
     if (!AreAnalysesValid(kAnalysisCFG)) {
       BuildCFG();
     }
@@ -377,30 +449,38 @@ class IRContext {
   }
 
   // Gets the loop descriptor for function |f|.
-  ir::LoopDescriptor* GetLoopDescriptor(const ir::Function* f);
+  LoopDescriptor* GetLoopDescriptor(const Function* f);
 
   // Gets the dominator analysis for function |f|.
-  opt::DominatorAnalysis* GetDominatorAnalysis(const ir::Function* f,
-                                               const ir::CFG&);
+  DominatorAnalysis* GetDominatorAnalysis(const Function* f);
 
   // Gets the postdominator analysis for function |f|.
-  opt::PostDominatorAnalysis* GetPostDominatorAnalysis(const ir::Function* f,
-                                                       const ir::CFG&);
+  PostDominatorAnalysis* GetPostDominatorAnalysis(const Function* f);
 
   // Remove the dominator tree of |f| from the cache.
-  inline void RemoveDominatorAnalysis(const ir::Function* f) {
+  inline void RemoveDominatorAnalysis(const Function* f) {
     dominator_trees_.erase(f);
   }
 
   // Remove the postdominator tree of |f| from the cache.
-  inline void RemovePostDominatorAnalysis(const ir::Function* f) {
+  inline void RemovePostDominatorAnalysis(const Function* f) {
     post_dominator_trees_.erase(f);
   }
 
-  // Return the next available SSA id and increment it.
-  inline uint32_t TakeNextId() { return module()->TakeNextIdBound(); }
+  // Return the next available SSA id and increment it.  Returns 0 if the
+  // maximum SSA id has been reached.
+  inline uint32_t TakeNextId() {
+    uint32_t next_id = module()->TakeNextIdBound();
+    if (next_id == 0) {
+      if (consumer()) {
+        std::string message = "ID overflow. Try running compact-ids.";
+        consumer()(SPV_MSG_ERROR, "", {0, 0, 0}, message.c_str());
+      }
+    }
+    return next_id;
+  }
 
-  opt::FeatureManager* get_feature_mgr() {
+  FeatureManager* get_feature_mgr() {
     if (!feature_mgr_.get()) {
       AnalyzeFeatures();
     }
@@ -408,16 +488,82 @@ class IRContext {
   }
 
   // Returns the grammar for this context.
-  const libspirv::AssemblyGrammar& grammar() const { return grammar_; }
+  const AssemblyGrammar& grammar() const { return grammar_; }
 
   // If |inst| has not yet been analysed by the def-use manager, then analyse
   // its definitions and uses.
   inline void UpdateDefUse(Instruction* inst);
 
+  const InstructionFolder& get_instruction_folder() {
+    if (!inst_folder_) {
+      inst_folder_ = MakeUnique<InstructionFolder>(this);
+    }
+    return *inst_folder_;
+  }
+
+  uint32_t max_id_bound() const { return max_id_bound_; }
+  void set_max_id_bound(uint32_t new_bound) { max_id_bound_ = new_bound; }
+
+  bool preserve_bindings() const { return preserve_bindings_; }
+  void set_preserve_bindings(bool should_preserve_bindings) {
+    preserve_bindings_ = should_preserve_bindings;
+  }
+
+  bool preserve_spec_constants() const { return preserve_spec_constants_; }
+  void set_preserve_spec_constants(bool should_preserve_spec_constants) {
+    preserve_spec_constants_ = should_preserve_spec_constants;
+  }
+
+  // Return id of input variable only decorated with |builtin|, if in module.
+  // Create variable and return its id otherwise. If builtin not currently
+  // supported, return 0.
+  uint32_t GetBuiltinInputVarId(uint32_t builtin);
+
+  // Returns the function whose id is |id|, if one exists.  Returns |nullptr|
+  // otherwise.
+  Function* GetFunction(uint32_t id) {
+    if (!AreAnalysesValid(kAnalysisIdToFuncMapping)) {
+      BuildIdToFuncMapping();
+    }
+    auto entry = id_to_func_.find(id);
+    return (entry != id_to_func_.end()) ? entry->second : nullptr;
+  }
+
+  Function* GetFunction(Instruction* inst) {
+    if (inst->opcode() != SpvOpFunction) {
+      return nullptr;
+    }
+    return GetFunction(inst->result_id());
+  }
+
+  // Add to |todo| all ids of functions called in |func|.
+  void AddCalls(const Function* func, std::queue<uint32_t>* todo);
+
+  // Applies |pfn| to every function in the call trees that are rooted at the
+  // entry points.  Returns true if any call |pfn| returns true.  By convention
+  // |pfn| should return true if it modified the module.
+  bool ProcessEntryPointCallTree(ProcessFunction& pfn);
+
+  // Applies |pfn| to every function in the call trees rooted at the entry
+  // points and exported functions.  Returns true if any call |pfn| returns
+  // true.  By convention |pfn| should return true if it modified the module.
+  bool ProcessReachableCallTree(ProcessFunction& pfn);
+
+  // Applies |pfn| to every function in the call trees rooted at the elements of
+  // |roots|.  Returns true if any call to |pfn| returns true.  By convention
+  // |pfn| should return true if it modified the module.  After returning
+  // |roots| will be empty.
+  bool ProcessCallTreeFromRoots(ProcessFunction& pfn,
+                                std::queue<uint32_t>* roots);
+
+  // Emmits a error message to the message consumer indicating the error
+  // described by |message| occurred in |inst|.
+  void EmitErrorMessage(std::string message, Instruction* inst);
+
  private:
   // Builds the def-use manager from scratch, even if it was already valid.
   void BuildDefUseManager() {
-    def_use_mgr_.reset(new opt::analysis::DefUseManager(module()));
+    def_use_mgr_ = MakeUnique<analysis::DefUseManager>(module());
     valid_analyses_ = valid_analyses_ | kAnalysisDefUse;
   }
 
@@ -426,7 +572,7 @@ class IRContext {
     instr_to_block_.clear();
     for (auto& fn : *module_) {
       for (auto& block : fn) {
-        block.ForEachInst([this, &block](ir::Instruction* inst) {
+        block.ForEachInst([this, &block](Instruction* inst) {
           instr_to_block_[inst] = &block;
         });
       }
@@ -434,16 +580,64 @@ class IRContext {
     valid_analyses_ = valid_analyses_ | kAnalysisInstrToBlockMapping;
   }
 
+  // Builds the instruction-function map for the whole module.
+  void BuildIdToFuncMapping() {
+    id_to_func_.clear();
+    for (auto& fn : *module_) {
+      id_to_func_[fn.result_id()] = &fn;
+    }
+    valid_analyses_ = valid_analyses_ | kAnalysisIdToFuncMapping;
+  }
+
   void BuildDecorationManager() {
-    decoration_mgr_.reset(new opt::analysis::DecorationManager(module()));
+    decoration_mgr_ = MakeUnique<analysis::DecorationManager>(module());
     valid_analyses_ = valid_analyses_ | kAnalysisDecorations;
   }
 
   void BuildCFG() {
-    cfg_.reset(new ir::CFG(module()));
+    cfg_ = MakeUnique<CFG>(module());
     valid_analyses_ = valid_analyses_ | kAnalysisCFG;
   }
 
+  void BuildScalarEvolutionAnalysis() {
+    scalar_evolution_analysis_ = MakeUnique<ScalarEvolutionAnalysis>(this);
+    valid_analyses_ = valid_analyses_ | kAnalysisScalarEvolution;
+  }
+
+  // Builds the liveness analysis from scratch, even if it was already valid.
+  void BuildRegPressureAnalysis() {
+    reg_pressure_ = MakeUnique<LivenessAnalysis>(this);
+    valid_analyses_ = valid_analyses_ | kAnalysisRegisterPressure;
+  }
+
+  // Builds the value number table analysis from scratch, even if it was already
+  // valid.
+  void BuildValueNumberTable() {
+    vn_table_ = MakeUnique<ValueNumberTable>(this);
+    valid_analyses_ = valid_analyses_ | kAnalysisValueNumberTable;
+  }
+
+  // Builds the structured CFG analysis from scratch, even if it was already
+  // valid.
+  void BuildStructuredCFGAnalysis() {
+    struct_cfg_analysis_ = MakeUnique<StructuredCFGAnalysis>(this);
+    valid_analyses_ = valid_analyses_ | kAnalysisStructuredCFG;
+  }
+
+  // Builds the constant manager from scratch, even if it was already
+  // valid.
+  void BuildConstantManager() {
+    constant_mgr_ = MakeUnique<analysis::ConstantManager>(this);
+    valid_analyses_ = valid_analyses_ | kAnalysisConstants;
+  }
+
+  // Builds the type manager from scratch, even if it was already
+  // valid.
+  void BuildTypeManager() {
+    type_mgr_ = MakeUnique<analysis::TypeManager>(consumer(), this);
+    valid_analyses_ = valid_analyses_ | kAnalysisTypes;
+  }
+
   // Removes all computed dominator and post-dominator trees. This will force
   // the context to rebuild the trees on demand.
   void ResetDominatorAnalysis() {
@@ -460,9 +654,16 @@ class IRContext {
     valid_analyses_ = valid_analyses_ | kAnalysisLoopAnalysis;
   }
 
+  // Removes all computed loop descriptors.
+  void ResetBuiltinAnalysis() {
+    // Clear the cache.
+    builtin_var_id_map_.clear();
+    valid_analyses_ = valid_analyses_ | kAnalysisBuiltinVarId;
+  }
+
   // Analyzes the features in the owned module. Builds the manager if required.
   void AnalyzeFeatures() {
-    feature_mgr_.reset(new opt::FeatureManager(grammar_));
+    feature_mgr_ = MakeUnique<FeatureManager>(grammar_);
     feature_mgr_->Analyze(module());
   }
 
@@ -474,17 +675,28 @@ class IRContext {
   void AddCombinatorsForCapability(uint32_t capability);
 
   // Add the combinator opcode for the given extension to combinator_ops_.
-  void AddCombinatorsForExtension(ir::Instruction* extension);
+  void AddCombinatorsForExtension(Instruction* extension);
 
   // Remove |inst| from |id_to_name_| if it is in map.
   void RemoveFromIdToName(const Instruction* inst);
 
+  // Returns true if it is suppose to be valid but it is incorrect.  Returns
+  // true if the cfg is invalidated.
+  bool CheckCFG();
+
+  // Return id of input variable only decorated with |builtin|, if in module.
+  // Return 0 otherwise.
+  uint32_t FindBuiltinInputVar(uint32_t builtin);
+
+  // Add |var_id| to all entry points in module.
+  void AddVarToEntryPoints(uint32_t var_id);
+
   // The SPIR-V syntax context containing grammar tables for opcodes and
   // operands.
   spv_context syntax_context_;
 
   // Auxiliary object for querying SPIR-V grammar facts.
-  libspirv::AssemblyGrammar grammar_;
+  AssemblyGrammar grammar_;
 
   // An unique identifier for instructions in |module_|. Can be used to order
   // instructions in a container.
@@ -497,21 +709,28 @@ class IRContext {
   std::unique_ptr<Module> module_;
 
   // A message consumer for diagnostics.
-  spvtools::MessageConsumer consumer_;
+  MessageConsumer consumer_;
 
   // The def-use manager for |module_|.
-  std::unique_ptr<opt::analysis::DefUseManager> def_use_mgr_;
+  std::unique_ptr<analysis::DefUseManager> def_use_mgr_;
 
   // The instruction decoration manager for |module_|.
-  std::unique_ptr<opt::analysis::DecorationManager> decoration_mgr_;
-  std::unique_ptr<opt::FeatureManager> feature_mgr_;
+  std::unique_ptr<analysis::DecorationManager> decoration_mgr_;
+  std::unique_ptr<FeatureManager> feature_mgr_;
 
-  // A map from instructions the the basic block they belong to. This mapping is
+  // A map from instructions to the basic block they belong to. This mapping is
   // built on-demand when get_instr_block() is called.
   //
   // NOTE: Do not traverse this map. Ever. Use the function and basic block
   // iterators to traverse instructions.
-  std::unordered_map<ir::Instruction*, ir::BasicBlock*> instr_to_block_;
+  std::unordered_map<Instruction*, BasicBlock*> instr_to_block_;
+
+  // A map from ids to the function they define. This mapping is
+  // built on-demand when GetFunction() is called.
+  //
+  // NOTE: Do not traverse this map. Ever. Use the function and basic block
+  // iterators to traverse instructions.
+  std::unordered_map<uint32_t, Function*> id_to_func_;
 
   // A bitset indicating which analyes are currently valid.
   Analysis valid_analyses_;
@@ -520,53 +739,76 @@ class IRContext {
   // without side-effect.
   std::unordered_map<uint32_t, std::unordered_set<uint32_t>> combinator_ops_;
 
+  // Opcodes of shader capability core executable instructions
+  // without side-effect.
+  std::unordered_map<uint32_t, uint32_t> builtin_var_id_map_;
+
   // The CFG for all the functions in |module_|.
-  std::unique_ptr<ir::CFG> cfg_;
+  std::unique_ptr<CFG> cfg_;
 
   // Each function in the module will create its own dominator tree. We cache
   // the result so it doesn't need to be rebuilt each time.
-  std::map<const ir::Function*, opt::DominatorAnalysis> dominator_trees_;
-  std::map<const ir::Function*, opt::PostDominatorAnalysis>
-      post_dominator_trees_;
+  std::map<const Function*, DominatorAnalysis> dominator_trees_;
+  std::map<const Function*, PostDominatorAnalysis> post_dominator_trees_;
 
   // Cache of loop descriptors for each function.
-  std::unordered_map<const ir::Function*, ir::LoopDescriptor> loop_descriptors_;
+  std::unordered_map<const Function*, LoopDescriptor> loop_descriptors_;
 
   // Constant manager for |module_|.
-  std::unique_ptr<opt::analysis::ConstantManager> constant_mgr_;
+  std::unique_ptr<analysis::ConstantManager> constant_mgr_;
 
   // Type manager for |module_|.
-  std::unique_ptr<opt::analysis::TypeManager> type_mgr_;
+  std::unique_ptr<analysis::TypeManager> type_mgr_;
 
   // A map from an id to its corresponding OpName and OpMemberName instructions.
   std::unique_ptr<std::multimap<uint32_t, Instruction*>> id_to_name_;
+
+  // The cache scalar evolution analysis node.
+  std::unique_ptr<ScalarEvolutionAnalysis> scalar_evolution_analysis_;
+
+  // The liveness analysis |module_|.
+  std::unique_ptr<LivenessAnalysis> reg_pressure_;
+
+  std::unique_ptr<ValueNumberTable> vn_table_;
+
+  std::unique_ptr<InstructionFolder> inst_folder_;
+
+  std::unique_ptr<StructuredCFGAnalysis> struct_cfg_analysis_;
+
+  // The maximum legal value for the id bound.
+  uint32_t max_id_bound_;
+
+  // Whether all bindings within |module_| should be preserved.
+  bool preserve_bindings_;
+
+  // Whether all specialization constants within |module_|
+  // should be preserved.
+  bool preserve_spec_constants_;
 };
 
-inline ir::IRContext::Analysis operator|(ir::IRContext::Analysis lhs,
-                                         ir::IRContext::Analysis rhs) {
-  return static_cast<ir::IRContext::Analysis>(static_cast<int>(lhs) |
-                                              static_cast<int>(rhs));
+inline IRContext::Analysis operator|(IRContext::Analysis lhs,
+                                     IRContext::Analysis rhs) {
+  return static_cast<IRContext::Analysis>(static_cast<int>(lhs) |
+                                          static_cast<int>(rhs));
 }
 
-inline ir::IRContext::Analysis& operator|=(ir::IRContext::Analysis& lhs,
-                                           ir::IRContext::Analysis rhs) {
-  lhs = static_cast<ir::IRContext::Analysis>(static_cast<int>(lhs) |
-                                             static_cast<int>(rhs));
+inline IRContext::Analysis& operator|=(IRContext::Analysis& lhs,
+                                       IRContext::Analysis rhs) {
+  lhs = static_cast<IRContext::Analysis>(static_cast<int>(lhs) |
+                                         static_cast<int>(rhs));
   return lhs;
 }
 
-inline ir::IRContext::Analysis operator<<(ir::IRContext::Analysis a,
-                                          int shift) {
-  return static_cast<ir::IRContext::Analysis>(static_cast<int>(a) << shift);
+inline IRContext::Analysis operator<<(IRContext::Analysis a, int shift) {
+  return static_cast<IRContext::Analysis>(static_cast<int>(a) << shift);
 }
 
-inline ir::IRContext::Analysis& operator<<=(ir::IRContext::Analysis& a,
-                                            int shift) {
-  a = static_cast<ir::IRContext::Analysis>(static_cast<int>(a) << shift);
+inline IRContext::Analysis& operator<<=(IRContext::Analysis& a, int shift) {
+  a = static_cast<IRContext::Analysis>(static_cast<int>(a) << shift);
   return a;
 }
 
-std::vector<Instruction*> spvtools::ir::IRContext::GetConstants() {
+std::vector<Instruction*> IRContext::GetConstants() {
   return module()->GetConstants();
 }
 
@@ -606,11 +848,11 @@ IteratorRange<Module::const_inst_iterator> IRContext::capabilities() const {
   return ((const Module*)module())->capabilities();
 }
 
-ir::Module::inst_iterator IRContext::types_values_begin() {
+Module::inst_iterator IRContext::types_values_begin() {
   return module()->types_values_begin();
 }
 
-ir::Module::inst_iterator IRContext::types_values_end() {
+Module::inst_iterator IRContext::types_values_end() {
   return module()->types_values_end();
 }
 
@@ -687,6 +929,9 @@ void IRContext::AddCapability(std::unique_ptr<Instruction>&& c) {
 }
 
 void IRContext::AddExtension(std::unique_ptr<Instruction>&& e) {
+  if (AreAnalysesValid(kAnalysisDefUse)) {
+    get_def_use_mgr()->AnalyzeInstDefUse(e.get());
+  }
   module()->AddExtension(std::move(e));
 }
 
@@ -728,14 +973,23 @@ void IRContext::AddAnnotationInst(std::unique_ptr<Instruction>&& a) {
   if (AreAnalysesValid(kAnalysisDecorations)) {
     get_decoration_mgr()->AddDecoration(a.get());
   }
+  if (AreAnalysesValid(kAnalysisDefUse)) {
+    get_def_use_mgr()->AnalyzeInstDefUse(a.get());
+  }
   module()->AddAnnotationInst(std::move(a));
 }
 
 void IRContext::AddType(std::unique_ptr<Instruction>&& t) {
   module()->AddType(std::move(t));
+  if (AreAnalysesValid(kAnalysisDefUse)) {
+    get_def_use_mgr()->AnalyzeInstDefUse(&*(--types_values_end()));
+  }
 }
 
 void IRContext::AddGlobalValue(std::unique_ptr<Instruction>&& v) {
+  if (AreAnalysesValid(kAnalysisDefUse)) {
+    get_def_use_mgr()->AnalyzeInstDefUse(&*v);
+  }
   module()->AddGlobalValue(std::move(v));
 }
 
@@ -756,7 +1010,7 @@ void IRContext::UpdateDefUse(Instruction* inst) {
 }
 
 void IRContext::BuildIdToNameMap() {
-  id_to_name_.reset(new std::multimap<uint32_t, Instruction*>());
+  id_to_name_ = MakeUnique<std::multimap<uint32_t, Instruction*>>();
   for (Instruction& debug_inst : debugs2()) {
     if (debug_inst.opcode() == SpvOpMemberName ||
         debug_inst.opcode() == SpvOpName) {
@@ -775,6 +1029,7 @@ IRContext::GetNames(uint32_t id) {
   return make_range(std::move(result.first), std::move(result.second));
 }
 
-}  // namespace ir
+}  // namespace opt
 }  // namespace spvtools
-#endif  // SPIRV_TOOLS_IR_CONTEXT_H
+
+#endif  // SOURCE_OPT_IR_CONTEXT_H_
