@@ -14,15 +14,20 @@
 
 // This file provides a class hierarchy for representing SPIR-V types.
 
-#ifndef LIBSPIRV_OPT_TYPES_H_
-#define LIBSPIRV_OPT_TYPES_H_
+#ifndef SOURCE_OPT_TYPES_H_
+#define SOURCE_OPT_TYPES_H_
 
+#include <map>
 #include <memory>
+#include <set>
 #include <string>
 #include <unordered_map>
+#include <unordered_set>
+#include <utility>
 #include <vector>
 
-#include "latest_version_spirv_header.h"
+#include "source/latest_version_spirv_header.h"
+#include "source/opt/instruction.h"
 #include "spirv-tools/libspirv.h"
 
 namespace spvtools {
@@ -52,11 +57,14 @@ class Pipe;
 class ForwardPointer;
 class PipeStorage;
 class NamedBarrier;
+class AccelerationStructureNV;
 
 // Abstract class for a SPIR-V type. It has a bunch of As<sublcass>() methods,
 // which is used as a way to probe the actual <subclass>.
 class Type {
  public:
+  typedef std::set<std::pair<const Pointer*, const Pointer*>> IsSameCache;
+
   // Available subtypes.
   //
   // When adding a new derived class of Type, please add an entry to the enum.
@@ -84,6 +92,7 @@ class Type {
     kForwardPointer,
     kPipeStorage,
     kNamedBarrier,
+    kAccelerationStructureNV,
   };
 
   Type(Kind k) : kind_(k) {}
@@ -100,7 +109,16 @@ class Type {
   bool HasSameDecorations(const Type* that) const;
   // Returns true if this type is exactly the same as |that| type, including
   // decorations.
-  virtual bool IsSame(const Type* that) const = 0;
+  bool IsSame(const Type* that) const {
+    IsSameCache seen;
+    return IsSameImpl(that, &seen);
+  }
+
+  // Returns true if this type is exactly the same as |that| type, including
+  // decorations.  |seen| is the set of |Pointer*| pair that are currently being
+  // compared in a parent call to |IsSameImpl|.
+  virtual bool IsSameImpl(const Type* that, IsSameCache* seen) const = 0;
+
   // Returns a human-readable string to represent this type.
   virtual std::string str() const = 0;
 
@@ -127,47 +145,58 @@ class Type {
   // TODO(alanbaker): Update this if variable pointers become a core feature.
   bool IsUniqueType(bool allowVariablePointers = false) const;
 
-// A bunch of methods for casting this type to a given type. Returns this if the
-// cast can be done, nullptr otherwise.
-#define DeclareCastMethod(target)                  \
-  virtual target* As##target() { return nullptr; } \
-  virtual const target* As##target() const { return nullptr; }
-  DeclareCastMethod(Void);
-  DeclareCastMethod(Bool);
-  DeclareCastMethod(Integer);
-  DeclareCastMethod(Float);
-  DeclareCastMethod(Vector);
-  DeclareCastMethod(Matrix);
-  DeclareCastMethod(Image);
-  DeclareCastMethod(Sampler);
-  DeclareCastMethod(SampledImage);
-  DeclareCastMethod(Array);
-  DeclareCastMethod(RuntimeArray);
-  DeclareCastMethod(Struct);
-  DeclareCastMethod(Opaque);
-  DeclareCastMethod(Pointer);
-  DeclareCastMethod(Function);
-  DeclareCastMethod(Event);
-  DeclareCastMethod(DeviceEvent);
-  DeclareCastMethod(ReserveId);
-  DeclareCastMethod(Queue);
-  DeclareCastMethod(Pipe);
-  DeclareCastMethod(ForwardPointer);
-  DeclareCastMethod(PipeStorage);
-  DeclareCastMethod(NamedBarrier);
-#undef DeclareCastMethod
-
   bool operator==(const Type& other) const;
 
   // Returns the hash value of this type.
   size_t HashValue() const;
 
   // Adds the necessary words to compute a hash value of this type to |words|.
-  void GetHashWords(std::vector<uint32_t>* words) const;
+  void GetHashWords(std::vector<uint32_t>* words) const {
+    std::unordered_set<const Type*> seen;
+    GetHashWords(words, &seen);
+  }
+
+  // Adds the necessary words to compute a hash value of this type to |words|.
+  void GetHashWords(std::vector<uint32_t>* words,
+                    std::unordered_set<const Type*>* seen) const;
 
   // Adds necessary extra words for a subtype to calculate a hash value into
   // |words|.
-  virtual void GetExtraHashWords(std::vector<uint32_t>* words) const = 0;
+  virtual void GetExtraHashWords(
+      std::vector<uint32_t>* words,
+      std::unordered_set<const Type*>* pSet) const = 0;
+
+// A bunch of methods for casting this type to a given type. Returns this if the
+// cast can be done, nullptr otherwise.
+// clang-format off
+#define DeclareCastMethod(target)                  \
+  virtual target* As##target() { return nullptr; } \
+  virtual const target* As##target() const { return nullptr; }
+  DeclareCastMethod(Void)
+  DeclareCastMethod(Bool)
+  DeclareCastMethod(Integer)
+  DeclareCastMethod(Float)
+  DeclareCastMethod(Vector)
+  DeclareCastMethod(Matrix)
+  DeclareCastMethod(Image)
+  DeclareCastMethod(Sampler)
+  DeclareCastMethod(SampledImage)
+  DeclareCastMethod(Array)
+  DeclareCastMethod(RuntimeArray)
+  DeclareCastMethod(Struct)
+  DeclareCastMethod(Opaque)
+  DeclareCastMethod(Pointer)
+  DeclareCastMethod(Function)
+  DeclareCastMethod(Event)
+  DeclareCastMethod(DeviceEvent)
+  DeclareCastMethod(ReserveId)
+  DeclareCastMethod(Queue)
+  DeclareCastMethod(Pipe)
+  DeclareCastMethod(ForwardPointer)
+  DeclareCastMethod(PipeStorage)
+  DeclareCastMethod(NamedBarrier)
+  DeclareCastMethod(AccelerationStructureNV)
+#undef DeclareCastMethod
 
  protected:
   // Decorations attached to this type. Each decoration is encoded as a vector
@@ -182,6 +211,7 @@ class Type {
 
   Kind kind_;
 };
+// clang-format on
 
 class Integer : public Type {
  public:
@@ -189,7 +219,6 @@ class Integer : public Type {
       : Type(kInteger), width_(w), signed_(is_signed) {}
   Integer(const Integer&) = default;
 
-  bool IsSame(const Type* that) const override;
   std::string str() const override;
 
   Integer* AsInteger() override { return this; }
@@ -197,9 +226,12 @@ class Integer : public Type {
   uint32_t width() const { return width_; }
   bool IsSigned() const { return signed_; }
 
-  void GetExtraHashWords(std::vector<uint32_t>* words) const override;
+  void GetExtraHashWords(std::vector<uint32_t>* words,
+                         std::unordered_set<const Type*>* pSet) const override;
 
  private:
+  bool IsSameImpl(const Type* that, IsSameCache*) const override;
+
   uint32_t width_;  // bit width
   bool signed_;     // true if this integer is signed
 };
@@ -209,25 +241,26 @@ class Float : public Type {
   Float(uint32_t w) : Type(kFloat), width_(w) {}
   Float(const Float&) = default;
 
-  bool IsSame(const Type* that) const override;
   std::string str() const override;
 
   Float* AsFloat() override { return this; }
   const Float* AsFloat() const override { return this; }
   uint32_t width() const { return width_; }
 
-  void GetExtraHashWords(std::vector<uint32_t>* words) const override;
+  void GetExtraHashWords(std::vector<uint32_t>* words,
+                         std::unordered_set<const Type*>* pSet) const override;
 
  private:
+  bool IsSameImpl(const Type* that, IsSameCache*) const override;
+
   uint32_t width_;  // bit width
 };
 
 class Vector : public Type {
  public:
-  Vector(Type* element_type, uint32_t count);
+  Vector(const Type* element_type, uint32_t count);
   Vector(const Vector&) = default;
 
-  bool IsSame(const Type* that) const override;
   std::string str() const override;
   const Type* element_type() const { return element_type_; }
   uint32_t element_count() const { return count_; }
@@ -235,19 +268,21 @@ class Vector : public Type {
   Vector* AsVector() override { return this; }
   const Vector* AsVector() const override { return this; }
 
-  void GetExtraHashWords(std::vector<uint32_t>* words) const override;
+  void GetExtraHashWords(std::vector<uint32_t>* words,
+                         std::unordered_set<const Type*>* pSet) const override;
 
  private:
-  Type* element_type_;
+  bool IsSameImpl(const Type* that, IsSameCache*) const override;
+
+  const Type* element_type_;
   uint32_t count_;
 };
 
 class Matrix : public Type {
  public:
-  Matrix(Type* element_type, uint32_t count);
+  Matrix(const Type* element_type, uint32_t count);
   Matrix(const Matrix&) = default;
 
-  bool IsSame(const Type* that) const override;
   std::string str() const override;
   const Type* element_type() const { return element_type_; }
   uint32_t element_count() const { return count_; }
@@ -255,10 +290,13 @@ class Matrix : public Type {
   Matrix* AsMatrix() override { return this; }
   const Matrix* AsMatrix() const override { return this; }
 
-  void GetExtraHashWords(std::vector<uint32_t>* words) const override;
+  void GetExtraHashWords(std::vector<uint32_t>* words,
+                         std::unordered_set<const Type*>* pSet) const override;
 
  private:
-  Type* element_type_;
+  bool IsSameImpl(const Type* that, IsSameCache*) const override;
+
+  const Type* element_type_;
   uint32_t count_;
 };
 
@@ -269,7 +307,6 @@ class Image : public Type {
         SpvAccessQualifier qualifier = SpvAccessQualifierReadOnly);
   Image(const Image&) = default;
 
-  bool IsSame(const Type* that) const override;
   std::string str() const override;
 
   Image* AsImage() override { return this; }
@@ -284,9 +321,12 @@ class Image : public Type {
   SpvImageFormat format() const { return format_; }
   SpvAccessQualifier access_qualifier() const { return access_qualifier_; }
 
-  void GetExtraHashWords(std::vector<uint32_t>* words) const override;
+  void GetExtraHashWords(std::vector<uint32_t>* words,
+                         std::unordered_set<const Type*>* pSet) const override;
 
  private:
+  bool IsSameImpl(const Type* that, IsSameCache*) const override;
+
   Type* sampled_type_;
   SpvDim dim_;
   uint32_t depth_;
@@ -302,7 +342,6 @@ class SampledImage : public Type {
   SampledImage(Type* image) : Type(kSampledImage), image_type_(image) {}
   SampledImage(const SampledImage&) = default;
 
-  bool IsSame(const Type* that) const override;
   std::string str() const override;
 
   SampledImage* AsSampledImage() override { return this; }
@@ -310,66 +349,103 @@ class SampledImage : public Type {
 
   const Type* image_type() const { return image_type_; }
 
-  void GetExtraHashWords(std::vector<uint32_t>* words) const override;
+  void GetExtraHashWords(std::vector<uint32_t>* words,
+                         std::unordered_set<const Type*>* pSet) const override;
 
  private:
+  bool IsSameImpl(const Type* that, IsSameCache*) const override;
   Type* image_type_;
 };
 
 class Array : public Type {
  public:
-  Array(Type* element_type, uint32_t length_id);
+  // Data about the length operand, that helps us distinguish between one
+  // array length and another.
+  struct LengthInfo {
+    // The result id of the instruction defining the length.
+    const uint32_t id;
+    enum Case : uint32_t {
+      kConstant = 0,
+      kConstantWithSpecId = 1,
+      kDefiningId = 2
+    };
+    // Extra words used to distinshish one array length and another.
+    //  - if OpConstant, then it's 0, then the words in the literal constant
+    //    value.
+    //  - if OpSpecConstant, then it's 1, then the SpecID decoration if there
+    //    is one, followed by the words in the literal constant value.
+    //    The spec might not be overridden, in which case we'll end up using
+    //    the literal value.
+    //  - Otherwise, it's an OpSpecConsant, and this 2, then the ID (again).
+    const std::vector<uint32_t> words;
+  };
+
+  // Constructs an array type with given element and length.  If the length
+  // is an OpSpecConstant, then |spec_id| should be its SpecId decoration.
+  Array(const Type* element_type, const LengthInfo& length_info_arg);
   Array(const Array&) = default;
 
-  bool IsSame(const Type* that) const override;
   std::string str() const override;
   const Type* element_type() const { return element_type_; }
-  uint32_t LengthId() const { return length_id_; }
+  uint32_t LengthId() const { return length_info_.id; }
+  const LengthInfo& length_info() const { return length_info_; }
 
   Array* AsArray() override { return this; }
   const Array* AsArray() const override { return this; }
 
-  void GetExtraHashWords(std::vector<uint32_t>* words) const override;
+  void GetExtraHashWords(std::vector<uint32_t>* words,
+                         std::unordered_set<const Type*>* pSet) const override;
+
+  void ReplaceElementType(const Type* element_type);
 
  private:
-  Type* element_type_;
-  uint32_t length_id_;
+  bool IsSameImpl(const Type* that, IsSameCache*) const override;
+
+  const Type* element_type_;
+  const LengthInfo length_info_;
 };
 
 class RuntimeArray : public Type {
  public:
-  RuntimeArray(Type* element_type);
+  RuntimeArray(const Type* element_type);
   RuntimeArray(const RuntimeArray&) = default;
 
-  bool IsSame(const Type* that) const override;
   std::string str() const override;
   const Type* element_type() const { return element_type_; }
 
   RuntimeArray* AsRuntimeArray() override { return this; }
   const RuntimeArray* AsRuntimeArray() const override { return this; }
 
-  void GetExtraHashWords(std::vector<uint32_t>* words) const override;
+  void GetExtraHashWords(std::vector<uint32_t>* words,
+                         std::unordered_set<const Type*>* pSet) const override;
+
+  void ReplaceElementType(const Type* element_type);
 
  private:
-  Type* element_type_;
+  bool IsSameImpl(const Type* that, IsSameCache*) const override;
+
+  const Type* element_type_;
 };
 
 class Struct : public Type {
  public:
-  Struct(const std::vector<Type*>& element_types);
+  Struct(const std::vector<const Type*>& element_types);
   Struct(const Struct&) = default;
 
   // Adds a decoration to the member at the given index.  The first word is the
   // decoration enum, and the remaining words, if any, are its operands.
   void AddMemberDecoration(uint32_t index, std::vector<uint32_t>&& decoration);
 
-  bool IsSame(const Type* that) const override;
   std::string str() const override;
-  const std::vector<Type*>& element_types() const { return element_types_; }
+  const std::vector<const Type*>& element_types() const {
+    return element_types_;
+  }
+  std::vector<const Type*>& element_types() { return element_types_; }
   bool decoration_empty() const override {
     return decorations_.empty() && element_decorations_.empty();
   }
-  const std::unordered_map<uint32_t, std::vector<std::vector<uint32_t>>>&
+
+  const std::map<uint32_t, std::vector<std::vector<uint32_t>>>&
   element_decorations() const {
     return element_decorations_;
   }
@@ -377,20 +453,24 @@ class Struct : public Type {
   Struct* AsStruct() override { return this; }
   const Struct* AsStruct() const override { return this; }
 
-  void GetExtraHashWords(std::vector<uint32_t>* words) const override;
+  void GetExtraHashWords(std::vector<uint32_t>* words,
+                         std::unordered_set<const Type*>* pSet) const override;
 
  private:
+  bool IsSameImpl(const Type* that, IsSameCache*) const override;
+
   void ClearDecorations() override {
     decorations_.clear();
     element_decorations_.clear();
   }
 
-  std::vector<Type*> element_types_;
+  std::vector<const Type*> element_types_;
   // We can attach decorations to struct members and that should not affect the
   // underlying element type. So we need an extra data structure here to keep
-  // track of element type decorations.
-  std::unordered_map<uint32_t, std::vector<std::vector<uint32_t>>>
-      element_decorations_;
+  // track of element type decorations.  They must be stored in an ordered map
+  // because |GetExtraHashWords| will traverse the structure.  It must have a
+  // fixed order in order to hash to the same value every time.
+  std::map<uint32_t, std::vector<std::vector<uint32_t>>> element_decorations_;
 };
 
 class Opaque : public Type {
@@ -398,7 +478,6 @@ class Opaque : public Type {
   Opaque(std::string n) : Type(kOpaque), name_(std::move(n)) {}
   Opaque(const Opaque&) = default;
 
-  bool IsSame(const Type* that) const override;
   std::string str() const override;
 
   Opaque* AsOpaque() override { return this; }
@@ -406,18 +485,20 @@ class Opaque : public Type {
 
   const std::string& name() const { return name_; }
 
-  void GetExtraHashWords(std::vector<uint32_t>* words) const override;
+  void GetExtraHashWords(std::vector<uint32_t>* words,
+                         std::unordered_set<const Type*>* pSet) const override;
 
  private:
+  bool IsSameImpl(const Type* that, IsSameCache*) const override;
+
   std::string name_;
 };
 
 class Pointer : public Type {
  public:
-  Pointer(Type* pointee, SpvStorageClass sc);
+  Pointer(const Type* pointee, SpvStorageClass sc);
   Pointer(const Pointer&) = default;
 
-  bool IsSame(const Type* that) const override;
   std::string str() const override;
   const Type* pointee_type() const { return pointee_type_; }
   SpvStorageClass storage_class() const { return storage_class_; }
@@ -425,32 +506,43 @@ class Pointer : public Type {
   Pointer* AsPointer() override { return this; }
   const Pointer* AsPointer() const override { return this; }
 
-  void GetExtraHashWords(std::vector<uint32_t>* words) const override;
+  void GetExtraHashWords(std::vector<uint32_t>* words,
+                         std::unordered_set<const Type*>* pSet) const override;
+
+  void SetPointeeType(const Type* type);
 
  private:
-  Type* pointee_type_;
+  bool IsSameImpl(const Type* that, IsSameCache*) const override;
+
+  const Type* pointee_type_;
   SpvStorageClass storage_class_;
 };
 
 class Function : public Type {
  public:
-  Function(Type* ret_type, const std::vector<Type*>& params);
+  Function(Type* ret_type, const std::vector<const Type*>& params);
+  Function(Type* ret_type, std::vector<const Type*>& params);
   Function(const Function&) = default;
 
-  bool IsSame(const Type* that) const override;
   std::string str() const override;
 
   Function* AsFunction() override { return this; }
   const Function* AsFunction() const override { return this; }
 
   const Type* return_type() const { return return_type_; }
-  const std::vector<Type*>& param_types() const { return param_types_; }
+  const std::vector<const Type*>& param_types() const { return param_types_; }
+  std::vector<const Type*>& param_types() { return param_types_; }
 
-  void GetExtraHashWords(std::vector<uint32_t>* words) const override;
+  void GetExtraHashWords(std::vector<uint32_t>* words,
+                         std::unordered_set<const Type*>*) const override;
+
+  void SetReturnType(const Type* type);
 
  private:
-  Type* return_type_;
-  std::vector<Type*> param_types_;
+  bool IsSameImpl(const Type* that, IsSameCache*) const override;
+
+  const Type* return_type_;
+  std::vector<const Type*> param_types_;
 };
 
 class Pipe : public Type {
@@ -459,7 +551,6 @@ class Pipe : public Type {
       : Type(kPipe), access_qualifier_(qualifier) {}
   Pipe(const Pipe&) = default;
 
-  bool IsSame(const Type* that) const override;
   std::string str() const override;
 
   Pipe* AsPipe() override { return this; }
@@ -467,9 +558,12 @@ class Pipe : public Type {
 
   SpvAccessQualifier access_qualifier() const { return access_qualifier_; }
 
-  void GetExtraHashWords(std::vector<uint32_t>* words) const override;
+  void GetExtraHashWords(std::vector<uint32_t>* words,
+                         std::unordered_set<const Type*>* pSet) const override;
 
  private:
+  bool IsSameImpl(const Type* that, IsSameCache*) const override;
+
   SpvAccessQualifier access_qualifier_;
 };
 
@@ -483,40 +577,45 @@ class ForwardPointer : public Type {
   ForwardPointer(const ForwardPointer&) = default;
 
   uint32_t target_id() const { return target_id_; }
-  void SetTargetPointer(Pointer* pointer) { pointer_ = pointer; }
+  void SetTargetPointer(const Pointer* pointer) { pointer_ = pointer; }
   SpvStorageClass storage_class() const { return storage_class_; }
   const Pointer* target_pointer() const { return pointer_; }
 
-  bool IsSame(const Type* that) const override;
   std::string str() const override;
 
   ForwardPointer* AsForwardPointer() override { return this; }
   const ForwardPointer* AsForwardPointer() const override { return this; }
 
-  void GetExtraHashWords(std::vector<uint32_t>* words) const override;
+  void GetExtraHashWords(std::vector<uint32_t>* words,
+                         std::unordered_set<const Type*>* pSet) const override;
 
  private:
+  bool IsSameImpl(const Type* that, IsSameCache*) const override;
+
   uint32_t target_id_;
   SpvStorageClass storage_class_;
-  Pointer* pointer_;
+  const Pointer* pointer_;
 };
 
-#define DefineParameterlessType(type, name)                          \
-  class type : public Type {                                         \
-   public:                                                           \
-    type() : Type(k##type) {}                                        \
-    type(const type&) = default;                                     \
-                                                                     \
-    bool IsSame(const Type* that) const override {                   \
-      return that->As##type() && HasSameDecorations(that);           \
-    }                                                                \
-    std::string str() const override { return #name; }               \
-                                                                     \
-    type* As##type() override { return this; }                       \
-    const type* As##type() const override { return this; }           \
-                                                                     \
-    void GetExtraHashWords(std::vector<uint32_t>*) const override {} \
-  };  // namespace analysis
+#define DefineParameterlessType(type, name)                                    \
+  class type : public Type {                                                   \
+   public:                                                                     \
+    type() : Type(k##type) {}                                                  \
+    type(const type&) = default;                                               \
+                                                                               \
+    std::string str() const override { return #name; }                         \
+                                                                               \
+    type* As##type() override { return this; }                                 \
+    const type* As##type() const override { return this; }                     \
+                                                                               \
+    void GetExtraHashWords(std::vector<uint32_t>*,                             \
+                           std::unordered_set<const Type*>*) const override {} \
+                                                                               \
+   private:                                                                    \
+    bool IsSameImpl(const Type* that, IsSameCache*) const override {           \
+      return that->As##type() && HasSameDecorations(that);                     \
+    }                                                                          \
+  }
 DefineParameterlessType(Void, void);
 DefineParameterlessType(Bool, bool);
 DefineParameterlessType(Sampler, sampler);
@@ -526,10 +625,11 @@ DefineParameterlessType(ReserveId, reserve_id);
 DefineParameterlessType(Queue, queue);
 DefineParameterlessType(PipeStorage, pipe_storage);
 DefineParameterlessType(NamedBarrier, named_barrier);
+DefineParameterlessType(AccelerationStructureNV, accelerationStructureNV);
 #undef DefineParameterlessType
 
 }  // namespace analysis
 }  // namespace opt
 }  // namespace spvtools
 
-#endif  // LIBSPIRV_OPT_TYPES_H_
+#endif  // SOURCE_OPT_TYPES_H_
